@@ -1,5 +1,4 @@
 // ANCHOR: imports_no_gc_work
-use crate::mmtk::MMTK;
 use crate::plan::global::BasePlan; //Modify
 use crate::plan::global::CommonPlan; // Add
 use crate::plan::global::GcStatus; // Add
@@ -11,8 +10,8 @@ use crate::plan::PlanConstraints;
 use crate::policy::copyspace::CopySpace; // Add
 use crate::policy::space::Space;
 use crate::scheduler::*; // Modify
-use crate::scheduler::gc_work::*; // Add
 use crate::util::alloc::allocators::AllocatorSelector;
+use crate::util::copy::*;
 use crate::util::heap::layout::heap_layout::Mmapper;
 use crate::util::heap::layout::heap_layout::VMMap;
 use crate::util::heap::layout::vm_layout_constants::{HEAP_END, HEAP_START};
@@ -27,18 +26,9 @@ use std::sync::atomic::{AtomicBool, Ordering}; // Add
 use std::sync::Arc;
 // ANCHOR_END: imports_no_gc_work
 
-// ANCHOR: imports_gc_work
-use super::gc_work::{MyGCCopyContext, MyGCProcessEdges}; // Add
-//ANCHOR_END: imports_gc_work
-
 // Remove #[allow(unused_imports)].
 // Remove handle_user_collection_request().
 
-
-pub type SelectedPlan<VM> = MyGC<VM>;
-
-pub const ALLOC_MyGC: AllocationSemantics = AllocationSemantics::Default; // Add
-
 // Modify
 // ANCHOR: plan_def
 pub struct MyGC<VM: VMBinding> {
@@ -66,17 +56,22 @@ impl<VM: VMBinding> Plan for MyGC<VM> {
         &MYGC_CONSTRAINTS
     }
 
-    // ANCHOR: create_worker_local
-    fn create_worker_local(
-        &self,
-        tls: VMWorkerThread,
-        mmtk: &'static MMTK<Self::VM>,
-    ) -> GCWorkerLocalPtr {
-        let mut c = MyGCCopyContext::new(mmtk);
-        c.init(tls);
-        GCWorkerLocalPtr::new(c)
+    // ANCHOR: create_copy_config
+    fn create_copy_config(&'static self) -> CopyConfig<Self::VM> {
+        use enum_map::enum_map;
+        CopyConfig {
+            copy_mapping: enum_map! {
+                CopySemantics::DefaultCopy => CopySelector::CopySpace(0),
+                _ => CopySelector::Unused,
+            },
+            space_mapping: vec![
+                // The tospace argument doesn't matter, we will rebind before a GC anyway.
+                (CopySelector::CopySpace(0), &self.copyspace0)
+            ],
+            constraints: &MYGC_CONSTRAINTS,
+        }
     }
-    // ANCHOR_END: create_worker_local
+    // ANCHOR_END: create_copy_config
 
     // Modify
     // ANCHOR: gc_init
@@ -84,9 +79,8 @@ impl<VM: VMBinding> Plan for MyGC<VM> {
         &mut self,
         heap_size: usize,
         vm_map: &'static VMMap,
-        scheduler: &Arc<GCWorkScheduler<VM>>,
     ) {
-        self.common.gc_init(heap_size, vm_map, scheduler);
+        self.common.gc_init(heap_size, vm_map);
         self.copyspace0.init(&vm_map);
         self.copyspace1.init(&vm_map);
     }
@@ -125,6 +119,13 @@ impl<VM: VMBinding> Plan for MyGC<VM> {
     }
     // ANCHOR_END: prepare
 
+    // Add
+    // ANCHOR: prepare_worker
+    fn prepare_worker(&self, worker: &mut GCWorker<VM>) {
+        unsafe { worker.get_copy_context_mut().copy[0].assume_init_mut() }.rebind(self.tospace());
+    }
+    // ANCHOR_END: prepare_worker
+
     // Modify
     // ANCHOR: release
     fn release(&mut self, tls: VMWorkerThread) {
@@ -135,17 +136,17 @@ impl<VM: VMBinding> Plan for MyGC<VM> {
 
     // Modify
     // ANCHOR: plan_get_collection_reserve
-    fn get_collection_reserve(&self) -> usize {
+    fn get_collection_reserved_pages(&self) -> usize {
         self.tospace().reserved_pages()
     }
     // ANCHOR_END: plan_get_collection_reserve
 
     // Modify
-    // ANCHOR: plan_get_pages_used
-    fn get_pages_used(&self) -> usize {
-        self.tospace().reserved_pages() + self.common.get_pages_used()
+    // ANCHOR: plan_get_used_pages
+    fn get_used_pages(&self) -> usize {
+        self.tospace().reserved_pages() + self.common.get_used_pages()
     }
-    // ANCHOR_END: plan_get_pages_used
+    // ANCHOR_END: plan_get_used_pages
 
     // Modify
     // ANCHOR: plan_base
@@ -169,7 +170,6 @@ impl<VM: VMBinding> MyGC<VM> {
         vm_map: &'static VMMap,
         mmapper: &'static Mmapper,
         options: Arc<UnsafeOptionsWrapper>,
-        _scheduler: &'static GCWorkScheduler<VM>,
     ) -> Self {
         // Modify
         let mut heap = HeapMeta::new(HEAP_START, HEAP_END);
