@@ -1,6 +1,5 @@
-use super::gc_work::{GenCopyCopyContext, GenCopyMatureGCWorkContext, GenCopyNurseryGCWorkContext};
+use super::gc_work::{GenCopyMatureGCWorkContext, GenCopyNurseryGCWorkContext};
 use super::mutator::ALLOCATOR_MAPPING;
-use crate::mmtk::MMTK;
 use crate::plan::generational::global::Gen;
 use crate::plan::global::BasePlan;
 use crate::plan::global::CommonPlan;
@@ -12,6 +11,7 @@ use crate::policy::copyspace::CopySpace;
 use crate::policy::space::Space;
 use crate::scheduler::*;
 use crate::util::alloc::allocators::AllocatorSelector;
+use crate::util::copy::*;
 use crate::util::heap::layout::heap_layout::Mmapper;
 use crate::util::heap::layout::heap_layout::VMMap;
 use crate::util::heap::layout::vm_layout_constants::{HEAP_END, HEAP_START};
@@ -25,8 +25,6 @@ use enum_map::EnumMap;
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::Arc;
 
-pub const ALLOC_SS: AllocationSemantics = AllocationSemantics::Default;
-
 pub struct GenCopy<VM: VMBinding> {
     pub gen: Gen<VM>,
     pub hi: AtomicBool,
@@ -43,14 +41,20 @@ impl<VM: VMBinding> Plan for GenCopy<VM> {
         &GENCOPY_CONSTRAINTS
     }
 
-    fn create_worker_local(
-        &self,
-        tls: VMWorkerThread,
-        mmtk: &'static MMTK<Self::VM>,
-    ) -> GCWorkerLocalPtr {
-        let mut c = GenCopyCopyContext::new(mmtk);
-        c.init(tls);
-        GCWorkerLocalPtr::new(c)
+    fn create_copy_config(&'static self) -> CopyConfig<Self::VM> {
+        use enum_map::enum_map;
+        CopyConfig {
+            copy_mapping: enum_map! {
+                CopySemantics::Mature => CopySelector::CopySpace(0),
+                CopySemantics::PromoteMature => CopySelector::CopySpace(0),
+                _ => CopySelector::Unused,
+            },
+            space_mapping: vec![
+                // The tospace argument doesn't matter, we will rebind before a GC anyway.
+                (CopySelector::CopySpace(0), self.tospace()),
+            ],
+            constraints: &GENCOPY_CONSTRAINTS,
+        }
     }
 
     fn collection_required(&self, space_full: bool, space: &dyn Space<Self::VM>) -> bool
@@ -68,13 +72,8 @@ impl<VM: VMBinding> Plan for GenCopy<VM> {
         self.gen.last_collection_full_heap()
     }
 
-    fn gc_init(
-        &mut self,
-        heap_size: usize,
-        vm_map: &'static VMMap,
-        scheduler: &Arc<GCWorkScheduler<VM>>,
-    ) {
-        self.gen.gc_init(heap_size, vm_map, scheduler);
+    fn gc_init(&mut self, heap_size: usize, vm_map: &'static VMMap) {
+        self.gen.gc_init(heap_size, vm_map);
         self.copyspace0.init(vm_map);
         self.copyspace1.init(vm_map);
     }
@@ -108,6 +107,10 @@ impl<VM: VMBinding> Plan for GenCopy<VM> {
         self.copyspace1.prepare(!hi);
     }
 
+    fn prepare_worker(&self, worker: &mut GCWorker<Self::VM>) {
+        unsafe { worker.get_copy_context_mut().copy[0].assume_init_mut() }.rebind(self.tospace());
+    }
+
     fn release(&mut self, tls: VMWorkerThread) {
         let full_heap = !self.is_current_gc_nursery();
         self.gen.release(tls);
@@ -119,18 +122,21 @@ impl<VM: VMBinding> Plan for GenCopy<VM> {
             .set_next_gc_full_heap(Gen::should_next_gc_be_full_heap(self));
     }
 
-    fn get_collection_reserve(&self) -> usize {
-        self.gen.get_collection_reserve() + self.tospace().reserved_pages()
+    fn get_collection_reserved_pages(&self) -> usize {
+        self.gen.get_collection_reserved_pages() + self.tospace().reserved_pages()
     }
 
-    fn get_pages_used(&self) -> usize {
-        self.gen.get_pages_used() + self.tospace().reserved_pages()
+    fn get_used_pages(&self) -> usize {
+        self.gen.get_used_pages() + self.tospace().reserved_pages()
     }
 
     /// Return the number of pages avilable for allocation. Assuming all future allocations goes to nursery.
-    fn get_pages_avail(&self) -> usize {
+    fn get_available_pages(&self) -> usize {
         // super.get_pages_avail() / 2 to reserve pages for copying
-        (self.get_total_pages() - self.get_pages_reserved()) >> 1
+        (self
+            .get_total_pages()
+            .saturating_sub(self.get_reserved_pages()))
+            >> 1
     }
 
     fn base(&self) -> &BasePlan<VM> {
@@ -213,7 +219,7 @@ impl<VM: VMBinding> GenCopy<VM> {
 
     fn request_full_heap_collection(&self) -> bool {
         self.gen
-            .request_full_heap_collection(self.get_total_pages(), self.get_pages_reserved())
+            .request_full_heap_collection(self.get_total_pages(), self.get_reserved_pages())
     }
 
     pub fn tospace(&self) -> &CopySpace<VM> {
