@@ -1,6 +1,4 @@
 use crate::plan::global::CommonPlan;
-use crate::plan::AllocationSemantics;
-use crate::plan::CopyContext;
 use crate::plan::Plan;
 use crate::plan::PlanConstraints;
 use crate::plan::TransitiveClosure;
@@ -8,6 +6,7 @@ use crate::policy::copyspace::CopySpace;
 use crate::policy::space::Space;
 use crate::scheduler::*;
 use crate::util::conversions;
+use crate::util::copy::CopySemantics;
 use crate::util::heap::layout::heap_layout::Mmapper;
 use crate::util::heap::layout::heap_layout::VMMap;
 use crate::util::heap::HeapMeta;
@@ -18,7 +17,6 @@ use crate::util::options::UnsafeOptionsWrapper;
 use crate::util::ObjectReference;
 use crate::util::VMWorkerThread;
 use crate::vm::VMBinding;
-
 use std::sync::atomic::AtomicBool;
 use std::sync::atomic::Ordering;
 use std::sync::Arc;
@@ -76,13 +74,8 @@ impl<VM: VMBinding> Gen<VM> {
     }
 
     /// Initialize Gen. This should be called by the gc_init() API call.
-    pub fn gc_init(
-        &mut self,
-        heap_size: usize,
-        vm_map: &'static VMMap,
-        scheduler: &Arc<GCWorkScheduler<VM>>,
-    ) {
-        self.common.gc_init(heap_size, vm_map, scheduler);
+    pub fn gc_init(&mut self, heap_size: usize, vm_map: &'static VMMap) {
+        self.common.gc_init(heap_size, vm_map);
         self.nursery.init(vm_map);
     }
 
@@ -109,7 +102,7 @@ impl<VM: VMBinding> Gen<VM> {
         space: &dyn Space<VM>,
     ) -> bool {
         let nursery_full = self.nursery.reserved_pages()
-            >= (conversions::bytes_to_pages_up(self.common.base.options.max_nursery));
+            >= (conversions::bytes_to_pages_up(*self.common.base.options.max_nursery));
         if nursery_full {
             return true;
         }
@@ -145,7 +138,7 @@ impl<VM: VMBinding> Gen<VM> {
             .base
             .user_triggered_collection
             .load(Ordering::SeqCst)
-            && self.common.base.options.full_heap_system_gc
+            && *self.common.base.options.full_heap_system_gc
         {
             // User triggered collection, and we force full heap for user triggered collection
             true
@@ -169,37 +162,37 @@ impl<VM: VMBinding> Gen<VM> {
     }
 
     /// Trace objects for spaces in generational and common plans for a full heap GC.
-    pub fn trace_object_full_heap<T: TransitiveClosure, C: CopyContext + GCWorkerLocal>(
+    pub fn trace_object_full_heap<T: TransitiveClosure>(
         &self,
         trace: &mut T,
         object: ObjectReference,
-        copy_context: &mut C,
+        worker: &mut GCWorker<VM>,
     ) -> ObjectReference {
         if self.nursery.in_space(object) {
-            return self.nursery.trace_object::<T, C>(
+            return self.nursery.trace_object::<T>(
                 trace,
                 object,
-                AllocationSemantics::Default,
-                copy_context,
+                CopySemantics::PromoteMature,
+                worker,
             );
         }
-        self.common.trace_object::<T, C>(trace, object)
+        self.common.trace_object::<T>(trace, object)
     }
 
     /// Trace objects for spaces in generational and common plans for a nursery GC.
-    pub fn trace_object_nursery<T: TransitiveClosure, C: CopyContext + GCWorkerLocal>(
+    pub fn trace_object_nursery<T: TransitiveClosure>(
         &self,
         trace: &mut T,
         object: ObjectReference,
-        copy_context: &mut C,
+        worker: &mut GCWorker<VM>,
     ) -> ObjectReference {
         // Evacuate nursery objects
         if self.nursery.in_space(object) {
-            return self.nursery.trace_object::<T, C>(
+            return self.nursery.trace_object::<T>(
                 trace,
                 object,
-                crate::plan::global::AllocationSemantics::Default,
-                copy_context,
+                CopySemantics::PromoteMature,
+                worker,
             );
         }
         // We may alloc large object into LOS as nursery objects. Trace them here.
@@ -216,7 +209,8 @@ impl<VM: VMBinding> Gen<VM> {
 
     /// Check a plan to see if the next GC should be a full heap GC.
     pub fn should_next_gc_be_full_heap(plan: &dyn Plan<VM = VM>) -> bool {
-        plan.get_pages_avail() < conversions::bytes_to_pages_up(plan.base().options.min_nursery)
+        plan.get_available_pages()
+            < conversions::bytes_to_pages_up(*plan.base().options.min_nursery)
     }
 
     /// Set next_gc_full_heap to the given value.
@@ -227,13 +221,13 @@ impl<VM: VMBinding> Gen<VM> {
 
     /// Get pages reserved for the collection by a generational plan. A generational plan should
     /// add their own reservatioin with the value returned by this method.
-    pub fn get_collection_reserve(&self) -> usize {
+    pub fn get_collection_reserved_pages(&self) -> usize {
         self.nursery.reserved_pages()
     }
 
     /// Get pages used by a generational plan. A generational plan should add their own used pages
     /// with the value returned by this method.
-    pub fn get_pages_used(&self) -> usize {
-        self.nursery.reserved_pages() + self.common.get_pages_used()
+    pub fn get_used_pages(&self) -> usize {
+        self.nursery.reserved_pages() + self.common.get_used_pages()
     }
 }
