@@ -1,6 +1,5 @@
-use super::gc_work::{
-    GenImmixCopyContext, GenImmixMatureGCWorkContext, GenImmixNurseryGCWorkContext,
-};
+use super::gc_work::GenImmixMatureGCWorkContext;
+use super::gc_work::GenImmixNurseryGCWorkContext;
 use crate::plan::generational::global::Gen;
 use crate::plan::global::BasePlan;
 use crate::plan::global::CommonPlan;
@@ -12,9 +11,8 @@ use crate::plan::PlanConstraints;
 use crate::policy::immix::ImmixSpace;
 use crate::policy::space::Space;
 use crate::scheduler::GCWorkScheduler;
-use crate::scheduler::GCWorkerLocalPtr;
-use crate::scheduler::*;
 use crate::util::alloc::allocators::AllocatorSelector;
+use crate::util::copy::*;
 use crate::util::heap::layout::heap_layout::Mmapper;
 use crate::util::heap::layout::heap_layout::VMMap;
 use crate::util::heap::layout::vm_layout_constants::{HEAP_END, HEAP_START};
@@ -22,7 +20,6 @@ use crate::util::heap::HeapMeta;
 use crate::util::options::UnsafeOptionsWrapper;
 use crate::util::VMWorkerThread;
 use crate::vm::*;
-use crate::MMTK;
 
 use enum_map::EnumMap;
 use std::sync::atomic::AtomicBool;
@@ -65,14 +62,17 @@ impl<VM: VMBinding> Plan for GenImmix<VM> {
         &GENIMMIX_CONSTRAINTS
     }
 
-    fn create_worker_local(
-        &self,
-        tls: VMWorkerThread,
-        mmtk: &'static MMTK<Self::VM>,
-    ) -> GCWorkerLocalPtr {
-        let mut c = GenImmixCopyContext::new(mmtk);
-        c.init(tls);
-        GCWorkerLocalPtr::new(c)
+    fn create_copy_config(&'static self) -> CopyConfig<Self::VM> {
+        use enum_map::enum_map;
+        CopyConfig {
+            copy_mapping: enum_map! {
+                CopySemantics::PromoteMature => CopySelector::Immix(0),
+                CopySemantics::Mature => CopySelector::Immix(0),
+                _ => CopySelector::Unused,
+            },
+            space_mapping: vec![(CopySelector::Immix(0), &self.immix)],
+            constraints: &GENIMMIX_CONSTRAINTS,
+        }
     }
 
     fn last_collection_was_exhaustive(&self) -> bool {
@@ -97,13 +97,8 @@ impl<VM: VMBinding> Plan for GenImmix<VM> {
         self.gen.collection_required(self, space_full, space)
     }
 
-    fn gc_init(
-        &mut self,
-        heap_size: usize,
-        vm_map: &'static VMMap,
-        scheduler: &Arc<GCWorkScheduler<VM>>,
-    ) {
-        self.gen.gc_init(heap_size, vm_map, scheduler);
+    fn gc_init(&mut self, heap_size: usize, vm_map: &'static VMMap) {
+        self.gen.gc_init(heap_size, vm_map);
         self.immix.init(vm_map);
     }
 
@@ -123,7 +118,7 @@ impl<VM: VMBinding> Plan for GenImmix<VM> {
                 true,
                 self.base().cur_collection_attempts.load(Ordering::SeqCst),
                 self.base().is_user_triggered_collection(),
-                self.base().options.full_heap_system_gc,
+                *self.base().options.full_heap_system_gc,
             )
         } else {
             false
@@ -170,18 +165,21 @@ impl<VM: VMBinding> Plan for GenImmix<VM> {
             .store(full_heap, Ordering::Relaxed);
     }
 
-    fn get_collection_reserve(&self) -> usize {
-        self.gen.get_collection_reserve() + self.immix.defrag_headroom_pages()
+    fn get_collection_reserved_pages(&self) -> usize {
+        self.gen.get_collection_reserved_pages() + self.immix.defrag_headroom_pages()
     }
 
-    fn get_pages_used(&self) -> usize {
-        self.gen.get_pages_used() + self.immix.reserved_pages()
+    fn get_used_pages(&self) -> usize {
+        self.gen.get_used_pages() + self.immix.reserved_pages()
     }
 
     /// Return the number of pages avilable for allocation. Assuming all future allocations goes to nursery.
-    fn get_pages_avail(&self) -> usize {
+    fn get_available_pages(&self) -> usize {
         // super.get_pages_avail() / 2 to reserve pages for copying
-        (self.get_total_pages() - self.get_pages_reserved()) >> 1
+        (self
+            .get_total_pages()
+            .saturating_sub(self.get_reserved_pages()))
+            >> 1
     }
 
     fn base(&self) -> &BasePlan<VM> {
@@ -253,6 +251,6 @@ impl<VM: VMBinding> GenImmix<VM> {
 
     fn request_full_heap_collection(&self) -> bool {
         self.gen
-            .request_full_heap_collection(self.get_total_pages(), self.get_pages_reserved())
+            .request_full_heap_collection(self.get_total_pages(), self.get_reserved_pages())
     }
 }
