@@ -1,5 +1,4 @@
 pub(super) use super::super::ALLOCATOR_MAPPING;
-use super::gc_work::GenImmixCopyContext;
 use crate::plan::barriers::ObjectRememberingBarrier;
 use crate::plan::generational::create_gen_space_mapping;
 use crate::plan::generational::gc_work::GenNurseryProcessEdges;
@@ -41,11 +40,10 @@ pub fn create_genimmix_mutator<VM: VMBinding>(
 
     Mutator {
         allocators: Allocators::<VM>::new(mutator_tls, &*mmtk.plan, &config.space_mapping),
-        barrier:
-            box ObjectRememberingBarrier::<GenNurseryProcessEdges<VM, GenImmixCopyContext<VM>>>::new(
-                mmtk,
-                *VM::VMObjectModel::GLOBAL_LOG_BIT_SPEC,
-            ),
+        barrier: box ObjectRememberingBarrier::<GenNurseryProcessEdges<VM>>::new(
+            mmtk,
+            *VM::VMObjectModel::GLOBAL_LOG_BIT_SPEC,
+        ),
         mutator_tls,
         config,
         plan: genimmix,
