@@ -1,6 +1,6 @@
 //! The global part of a plan implementation.
 
-use super::controller_collector_context::ControllerCollectorContext;
+use super::gc_requester::GCRequester;
 use super::PlanConstraints;
 use crate::mmtk::MMTK;
 use crate::plan::generational::global::Gen;
@@ -14,6 +14,7 @@ use crate::util::alloc::allocators::AllocatorSelector;
 #[cfg(feature = "analysis")]
 use crate::util::analysis::AnalysisManager;
 use crate::util::conversions::bytes_to_pages;
+use crate::util::copy::{CopyConfig, GCWorkerCopyContext};
 use crate::util::heap::layout::heap_layout::Mmapper;
 use crate::util::heap::layout::heap_layout::VMMap;
 use crate::util::heap::layout::map::Map;
@@ -24,100 +25,19 @@ use crate::util::metadata::side_metadata::SideMetadataSpec;
 use crate::util::options::PlanSelector;
 use crate::util::options::{Options, UnsafeOptionsWrapper};
 use crate::util::statistics::stats::Stats;
-use crate::util::{Address, ObjectReference};
+use crate::util::ObjectReference;
 use crate::util::{VMMutatorThread, VMWorkerThread};
 use crate::vm::*;
 use downcast_rs::Downcast;
 use enum_map::EnumMap;
-use std::marker::PhantomData;
 use std::sync::atomic::{AtomicBool, AtomicUsize, Ordering};
 use std::sync::{Arc, Mutex};
 
-/// A GC worker's context for copying GCs.
-/// Each GC plan should provide their implementation of a CopyContext.
-/// For non-copying GC, NoCopy can be used.
-pub trait CopyContext: 'static + Send {
-    type VM: VMBinding;
-    fn constraints(&self) -> &'static PlanConstraints;
-    fn init(&mut self, tls: VMWorkerThread);
-    fn prepare(&mut self);
-    fn release(&mut self);
-    fn alloc_copy(
-        &mut self,
-        original: ObjectReference,
-        bytes: usize,
-        align: usize,
-        offset: isize,
-        semantics: AllocationSemantics,
-    ) -> Address;
-    fn post_copy(
-        &mut self,
-        _obj: ObjectReference,
-        _tib: Address,
-        _bytes: usize,
-        _semantics: AllocationSemantics,
-    ) {
-    }
-    fn copy_check_allocator(
-        &self,
-        _from: ObjectReference,
-        bytes: usize,
-        align: usize,
-        semantics: AllocationSemantics,
-    ) -> AllocationSemantics {
-        let large = crate::util::alloc::allocator::get_maximum_aligned_size::<Self::VM>(
-            bytes,
-            align,
-            Self::VM::MIN_ALIGNMENT,
-        ) > self.constraints().max_non_los_copy_bytes;
-        if large {
-            AllocationSemantics::Los
-        } else {
-            semantics
-        }
-    }
-}
-
-pub struct NoCopy<VM: VMBinding>(PhantomData<VM>);
-
-impl<VM: VMBinding> CopyContext for NoCopy<VM> {
-    type VM = VM;
-
-    fn init(&mut self, _tls: VMWorkerThread) {}
-    fn constraints(&self) -> &'static PlanConstraints {
-        unreachable!()
-    }
-    fn prepare(&mut self) {}
-    fn release(&mut self) {}
-    fn alloc_copy(
-        &mut self,
-        _original: ObjectReference,
-        _bytes: usize,
-        _align: usize,
-        _offset: isize,
-        _semantics: AllocationSemantics,
-    ) -> Address {
-        unreachable!()
-    }
-}
-
-impl<VM: VMBinding> NoCopy<VM> {
-    pub fn new(_mmtk: &'static MMTK<VM>) -> Self {
-        Self(PhantomData)
-    }
-}
-
-impl<VM: VMBinding> GCWorkerLocal for NoCopy<VM> {
-    fn init(&mut self, tls: VMWorkerThread) {
-        CopyContext::init(self, tls);
-    }
-}
-
 pub fn create_mutator<VM: VMBinding>(
     tls: VMMutatorThread,
     mmtk: &'static MMTK<VM>,
 ) -> Box<Mutator<VM>> {
-    Box::new(match mmtk.options.plan {
+    Box::new(match *mmtk.options.plan {
         PlanSelector::NoGC => crate::plan::nogc::mutator::create_nogc_mutator(tls, &*mmtk.plan),
         PlanSelector::SemiSpace => {
             crate::plan::semispace::mutator::create_ss_mutator(tls, &*mmtk.plan)
@@ -174,6 +94,14 @@ pub fn create_plan<VM: VMBinding>(
     }
 }
 
+/// Create thread local GC worker.
+pub fn create_gc_worker_context<VM: VMBinding>(
+    tls: VMWorkerThread,
+    mmtk: &'static MMTK<VM>,
+) -> GCWorkerCopyContext<VM> {
+    GCWorkerCopyContext::<VM>::new(tls, &*mmtk.plan, mmtk.plan.create_copy_config())
+}
+
 /// A plan describes the global core functionality for all memory management schemes.
 /// All global MMTk plans should implement this trait.
 ///
@@ -203,11 +131,14 @@ pub trait Plan: 'static + Sync + Downcast {
     type VM: VMBinding;
 
     fn constraints(&self) -> &'static PlanConstraints;
-    fn create_worker_local(
-        &self,
-        tls: VMWorkerThread,
-        mmtk: &'static MMTK<Self::VM>,
-    ) -> GCWorkerLocalPtr;
+
+    /// Create a copy config for this plan. A copying GC plan MUST override this method,
+    /// and provide a valid config.
+    fn create_copy_config(&'static self) -> CopyConfig<Self::VM> {
+        // Use the empty default copy config for non copying GC.
+        CopyConfig::default()
+    }
+
     fn base(&self) -> &BasePlan<Self::VM>;
     fn schedule_collection(&'static self, _scheduler: &GCWorkScheduler<Self::VM>);
     fn common(&self) -> &CommonPlan<Self::VM> {
@@ -224,12 +155,7 @@ pub trait Plan: 'static + Sync + Downcast {
     }
 
     // unsafe because this can only be called once by the init thread
-    fn gc_init(
-        &mut self,
-        heap_size: usize,
-        vm_map: &'static VMMap,
-        scheduler: &Arc<GCWorkScheduler<Self::VM>>,
-    );
+    fn gc_init(&mut self, heap_size: usize, vm_map: &'static VMMap);
 
     fn get_allocator_mapping(&self) -> &'static EnumMap<AllocationSemantics, AllocatorSelector>;
 
@@ -263,7 +189,16 @@ pub trait Plan: 'static + Sync + Downcast {
             .load(Ordering::SeqCst)
     }
 
+    /// Prepare the plan before a GC. This is invoked in an initial step in the GC.
+    /// This is invoked once per GC by one worker thread. 'tls' is the worker thread that executes this method.
     fn prepare(&mut self, tls: VMWorkerThread);
+
+    /// Prepare a worker for a GC. Each worker has its own prepare method. This hook is for plan-specific
+    /// per-worker preparation. This method is invoked once per worker by the worker thread passed as the argument.
+    fn prepare_worker(&self, _worker: &mut GCWorker<Self::VM>) {}
+
+    /// Release the plan after a GC. This is invoked at the end of a GC when most GC work is finished.
+    /// This is invoked once per GC by one worker thread. 'tls' is the worker thread that executes this method.
     fn release(&mut self, tls: VMWorkerThread);
 
     fn poll(&self, space_full: bool, space: &dyn Space<Self::VM>) -> bool {
@@ -279,7 +214,7 @@ pub trait Plan: 'static + Sync + Downcast {
                 return false;
             }*/
             self.log_poll(space, "Triggering collection");
-            self.base().control_collector_context.request();
+            self.base().gc_requester.request();
             return true;
         }
 
@@ -313,32 +248,56 @@ pub trait Plan: 'static + Sync + Downcast {
      */
     fn collection_required(&self, space_full: bool, _space: &dyn Space<Self::VM>) -> bool;
 
-    fn get_pages_reserved(&self) -> usize {
-        self.get_pages_used() + self.get_collection_reserve()
+    // Note: The following methods are about page accounting. The default implementation should
+    // work fine for non-copying plans. For copying plans, the plan should override any of these methods
+    // if necessary.
+
+    /// Get the number of pages that are reserved, including used pages and pages that will
+    /// be used (e.g. for copying).
+    fn get_reserved_pages(&self) -> usize {
+        self.get_used_pages() + self.get_collection_reserved_pages()
     }
 
+    /// Get the total number of pages for the heap.
     fn get_total_pages(&self) -> usize {
         self.base().heap.get_total_pages()
     }
 
-    fn get_pages_avail(&self) -> usize {
-        self.get_total_pages() - self.get_pages_reserved()
+    /// Get the number of pages that are still available for use. The available pages
+    /// should always be positive or 0.
+    fn get_available_pages(&self) -> usize {
+        // It is possible that the reserved pages is larger than the total pages so we are doing
+        // a saturating substraction to make sure we return a non-negative number.
+        // For example,
+        // 1. our GC trigger checks if reserved pages is more than total pages.
+        // 2. when the heap is almost full of live objects (such as in the case of an OOM) and we are doing a copying GC, it is possible
+        //    the reserved pages is larger than total pages after the copying GC (the reserved pages after a GC
+        //    may be larger than the reserved pages before a GC, as we may end up using more memory for thread local
+        //    buffers for copy allocators).
+        self.get_total_pages()
+            .saturating_sub(self.get_reserved_pages())
     }
 
-    fn get_collection_reserve(&self) -> usize {
+    /// Get the number of pages that are reserved for collection. By default, we return 0.
+    /// For copying plans, they need to override this and calculate required pages to complete
+    /// a copying GC.
+    fn get_collection_reserved_pages(&self) -> usize {
         0
     }
 
-    fn get_pages_used(&self) -> usize;
+    /// Get the number of pages that are used.
+    fn get_used_pages(&self) -> usize;
+
+    /// Get the number of pages that are NOT used. This is clearly different from available pages.
+    /// Free pages are unused, but some of them may have been reserved for some reason.
+    fn get_free_pages(&self) -> usize {
+        self.get_total_pages() - self.get_used_pages()
+    }
 
     fn is_emergency_collection(&self) -> bool {
         self.base().emergency_collection.load(Ordering::Relaxed)
     }
 
-    fn get_free_pages(&self) -> usize {
-        self.get_total_pages() - self.get_pages_used()
-    }
-
     /// The application code has requested a collection.
     fn handle_user_collection_request(&self, tls: VMMutatorThread, force: bool) {
         self.base().handle_user_collection_request(tls, force)
@@ -397,7 +356,7 @@ pub struct BasePlan<VM: VMBinding> {
     pub max_collection_attempts: AtomicUsize,
     // Current collection attempt
     pub cur_collection_attempts: AtomicUsize,
-    pub control_collector_context: ControllerCollectorContext<VM>,
+    pub gc_requester: Arc<GCRequester<VM>>,
     pub stats: Stats,
     mmapper: &'static Mmapper,
     pub vm_map: &'static VMMap,
@@ -511,7 +470,7 @@ impl<VM: VMBinding> BasePlan<VM> {
                 vm_map,
                 mmapper,
                 &mut heap,
-                options.vm_space_size,
+                *options.vm_space_size,
                 constraints,
                 global_side_metadata_specs,
             ),
@@ -528,7 +487,7 @@ impl<VM: VMBinding> BasePlan<VM> {
             allocation_success: AtomicBool::new(false),
             max_collection_attempts: AtomicUsize::new(0),
             cur_collection_attempts: AtomicUsize::new(0),
-            control_collector_context: ControllerCollectorContext::new(),
+            gc_requester: Arc::new(GCRequester::new()),
             stats,
             mmapper,
             heap,
@@ -544,12 +503,7 @@ impl<VM: VMBinding> BasePlan<VM> {
         }
     }
 
-    pub fn gc_init(
-        &mut self,
-        heap_size: usize,
-        vm_map: &'static VMMap,
-        scheduler: &Arc<GCWorkScheduler<VM>>,
-    ) {
+    pub fn gc_init(&mut self, heap_size: usize, vm_map: &'static VMMap) {
         vm_map.boot();
         vm_map.finalize_static_space_map(
             self.heap.get_discontig_start(),
@@ -558,7 +512,6 @@ impl<VM: VMBinding> BasePlan<VM> {
         self.heap
             .total_pages
             .store(bytes_to_pages(heap_size), Ordering::Relaxed);
-        self.control_collector_context.init(scheduler);
 
         #[cfg(feature = "code_space")]
         self.code_space.init(vm_map);
@@ -575,11 +528,11 @@ impl<VM: VMBinding> BasePlan<VM> {
 
     /// The application code has requested a collection.
     pub fn handle_user_collection_request(&self, tls: VMMutatorThread, force: bool) {
-        if force || !self.options.ignore_system_g_c {
+        if force || !*self.options.ignore_system_g_c {
             info!("User triggering collection");
             self.user_triggered_collection
                 .store(true, Ordering::Relaxed);
-            self.control_collector_context.request();
+            self.gc_requester.request();
             VM::VMCollection::block_for_gc(tls);
         }
     }
@@ -592,7 +545,7 @@ impl<VM: VMBinding> BasePlan<VM> {
             .store(true, Ordering::Relaxed);
         self.internal_triggered_collection
             .store(true, Ordering::Relaxed);
-        self.control_collector_context.request();
+        self.gc_requester.request();
     }
 
     /// Reset collection state information.
@@ -608,7 +561,7 @@ impl<VM: VMBinding> BasePlan<VM> {
     }
 
     // Depends on what base spaces we use, unsync may be unused.
-    pub fn get_pages_used(&self) -> usize {
+    pub fn get_used_pages(&self) -> usize {
         // Depends on what base spaces we use, pages may be unchanged.
         #[allow(unused_mut)]
         let mut pages = 0;
@@ -628,7 +581,7 @@ impl<VM: VMBinding> BasePlan<VM> {
         pages
     }
 
-    pub fn trace_object<T: TransitiveClosure, C: CopyContext>(
+    pub fn trace_object<T: TransitiveClosure>(
         &self,
         _trace: &mut T,
         _object: ObjectReference,
@@ -802,21 +755,21 @@ impl<VM: VMBinding> BasePlan<VM> {
     /// we should do stress GC.
     pub fn is_stress_test_gc_enabled(&self) -> bool {
         use crate::util::constants::DEFAULT_STRESS_FACTOR;
-        self.options.stress_factor != DEFAULT_STRESS_FACTOR
-            || self.options.analysis_factor != DEFAULT_STRESS_FACTOR
+        *self.options.stress_factor != DEFAULT_STRESS_FACTOR
+            || *self.options.analysis_factor != DEFAULT_STRESS_FACTOR
     }
 
     /// Check if we should do precise stress test. If so, we need to check for stress GCs for every allocation.
     /// Otherwise, we only check in the allocation slow path.
     pub fn is_precise_stress(&self) -> bool {
-        self.options.precise_stress
+        *self.options.precise_stress
     }
 
     /// Check if we should do a stress GC now. If GC is initialized and the allocation bytes exceeds
     /// the stress factor, we should do a stress GC.
     pub fn should_do_stress_gc(&self) -> bool {
         self.initialized.load(Ordering::SeqCst)
-            && (self.allocation_bytes.load(Ordering::SeqCst) > self.options.stress_factor)
+            && (self.allocation_bytes.load(Ordering::SeqCst) > *self.options.stress_factor)
     }
 
     pub(super) fn collection_required<P: Plan>(
@@ -830,18 +783,20 @@ impl<VM: VMBinding> BasePlan<VM> {
             debug!(
                 "Stress GC: allocation_bytes = {}, stress_factor = {}",
                 self.allocation_bytes.load(Ordering::Relaxed),
-                self.options.stress_factor
+                *self.options.stress_factor
             );
             debug!("Doing stress GC");
             self.allocation_bytes.store(0, Ordering::SeqCst);
         }
 
         debug!(
-            "self.get_pages_reserved()={}, self.get_total_pages()={}",
-            plan.get_pages_reserved(),
+            "self.get_reserved_pages()={}, self.get_total_pages()={}",
+            plan.get_reserved_pages(),
             plan.get_total_pages()
         );
-        let heap_full = plan.get_pages_reserved() > plan.get_total_pages();
+        // Check if we reserved more pages (including the collection copy reserve)
+        // than the heap's total pages. In that case, we will have to do a GC.
+        let heap_full = plan.get_reserved_pages() > plan.get_total_pages();
 
         space_full || stress_force_gc || heap_full
     }
@@ -914,22 +869,17 @@ impl<VM: VMBinding> CommonPlan<VM> {
         }
     }
 
-    pub fn gc_init(
-        &mut self,
-        heap_size: usize,
-        vm_map: &'static VMMap,
-        scheduler: &Arc<GCWorkScheduler<VM>>,
-    ) {
-        self.base.gc_init(heap_size, vm_map, scheduler);
+    pub fn gc_init(&mut self, heap_size: usize, vm_map: &'static VMMap) {
+        self.base.gc_init(heap_size, vm_map);
         self.immortal.init(vm_map);
         self.los.init(vm_map);
     }
 
-    pub fn get_pages_used(&self) -> usize {
-        self.immortal.reserved_pages() + self.los.reserved_pages() + self.base.get_pages_used()
+    pub fn get_used_pages(&self) -> usize {
+        self.immortal.reserved_pages() + self.los.reserved_pages() + self.base.get_used_pages()
     }
 
-    pub fn trace_object<T: TransitiveClosure, C: CopyContext>(
+    pub fn trace_object<T: TransitiveClosure>(
         &self,
         trace: &mut T,
         object: ObjectReference,
@@ -942,7 +892,7 @@ impl<VM: VMBinding> CommonPlan<VM> {
             trace!("trace_object: object in los");
             return self.los.trace_object(trace, object);
         }
-        self.base.trace_object::<T, C>(trace, object)
+        self.base.trace_object::<T>(trace, object)
     }
 
     pub fn prepare(&mut self, tls: VMWorkerThread, full_heap: bool) {
