@@ -1,81 +1,12 @@
 use super::global::Immix;
-use crate::plan::PlanConstraints;
 use crate::policy::space::Space;
 use crate::scheduler::gc_work::*;
-use crate::scheduler::GCWorkerLocal;
-use crate::util::alloc::{Allocator, ImmixAllocator};
-use crate::util::object_forwarding;
+use crate::util::copy::CopySemantics;
 use crate::util::{Address, ObjectReference};
 use crate::vm::VMBinding;
 use crate::MMTK;
-use crate::{
-    plan::CopyContext,
-    util::opaque_pointer::{VMThread, VMWorkerThread},
-};
 use std::ops::{Deref, DerefMut};
 
-/// Immix copy allocator
-pub struct ImmixCopyContext<VM: VMBinding> {
-    immix: ImmixAllocator<VM>,
-}
-
-impl<VM: VMBinding> CopyContext for ImmixCopyContext<VM> {
-    type VM = VM;
-
-    fn constraints(&self) -> &'static PlanConstraints {
-        &super::global::IMMIX_CONSTRAINTS
-    }
-    fn init(&mut self, tls: VMWorkerThread) {
-        self.immix.tls = tls.0;
-    }
-    fn prepare(&mut self) {
-        self.immix.reset()
-    }
-    fn release(&mut self) {
-        self.immix.reset()
-    }
-    #[inline(always)]
-    fn alloc_copy(
-        &mut self,
-        _original: ObjectReference,
-        bytes: usize,
-        align: usize,
-        offset: isize,
-        _semantics: crate::AllocationSemantics,
-    ) -> Address {
-        self.immix.alloc(bytes, align, offset)
-    }
-    #[inline(always)]
-    fn post_copy(
-        &mut self,
-        obj: ObjectReference,
-        _tib: Address,
-        _bytes: usize,
-        _semantics: crate::AllocationSemantics,
-    ) {
-        object_forwarding::clear_forwarding_bits::<VM>(obj);
-    }
-}
-
-impl<VM: VMBinding> ImmixCopyContext<VM> {
-    pub fn new(mmtk: &'static MMTK<VM>) -> Self {
-        Self {
-            immix: ImmixAllocator::new(
-                VMThread::UNINITIALIZED,
-                Some(&mmtk.plan.downcast_ref::<Immix<VM>>().unwrap().immix_space),
-                &*mmtk.plan,
-                true,
-            ),
-        }
-    }
-}
-
-impl<VM: VMBinding> GCWorkerLocal for ImmixCopyContext<VM> {
-    fn init(&mut self, tls: VMWorkerThread) {
-        CopyContext::init(self, tls);
-    }
-}
-
 #[derive(Debug, PartialEq, Eq, Copy, Clone)]
 pub(in crate::plan) enum TraceKind {
     Fast,
@@ -86,7 +17,7 @@ pub(super) struct ImmixProcessEdges<VM: VMBinding, const KIND: TraceKind> {
     // Use a static ref to the specific plan to avoid overhead from dynamic dispatch or
     // downcast for each traced object.
     plan: &'static Immix<VM>,
-    base: ProcessEdgesBase<Self>,
+    base: ProcessEdgesBase<VM>,
 }
 
 impl<VM: VMBinding, const KIND: TraceKind> ImmixProcessEdges<VM, KIND> {
@@ -132,14 +63,12 @@ impl<VM: VMBinding, const KIND: TraceKind> ProcessEdgesWork for ImmixProcessEdge
                 self.immix().immix_space.trace_object(
                     self,
                     object,
-                    super::global::ALLOC_IMMIX,
-                    unsafe { self.worker().local::<ImmixCopyContext<VM>>() },
+                    CopySemantics::DefaultCopy,
+                    self.worker(),
                 )
             }
         } else {
-            self.immix()
-                .common
-                .trace_object::<Self, ImmixCopyContext<VM>>(self, object)
+            self.immix().common.trace_object::<Self>(self, object)
         }
     }
 
@@ -154,7 +83,7 @@ impl<VM: VMBinding, const KIND: TraceKind> ProcessEdgesWork for ImmixProcessEdge
 }
 
 impl<VM: VMBinding, const KIND: TraceKind> Deref for ImmixProcessEdges<VM, KIND> {
-    type Target = ProcessEdgesBase<Self>;
+    type Target = ProcessEdgesBase<VM>;
     #[inline]
     fn deref(&self) -> &Self::Target {
         &self.base
@@ -176,6 +105,5 @@ impl<VM: VMBinding, const KIND: TraceKind> crate::scheduler::GCWorkContext
 {
     type VM = VM;
     type PlanType = Immix<VM>;
-    type CopyContextType = ImmixCopyContext<VM>;
     type ProcessEdgesWorkType = ImmixProcessEdges<VM, KIND>;
 }
