@@ -1,7 +1,10 @@
 use super::PageProtect;
 use crate::plan::mutator_context::Mutator;
 use crate::plan::mutator_context::MutatorConfig;
-use crate::plan::AllocationSemantics as AllocationType;
+use crate::plan::mutator_context::{
+    create_allocator_mapping, create_space_mapping, ReservedAllocators,
+};
+use crate::plan::AllocationSemantics;
 use crate::plan::Plan;
 use crate::util::alloc::allocators::{AllocatorSelector, Allocators};
 use crate::vm::VMBinding;
@@ -9,7 +12,6 @@ use crate::{
     plan::barriers::NoBarrier,
     util::opaque_pointer::{VMMutatorThread, VMWorkerThread},
 };
-use enum_map::enum_map;
 use enum_map::EnumMap;
 
 /// Prepare mutator. Do nothing.
@@ -18,11 +20,16 @@ fn pp_mutator_prepare<VM: VMBinding>(_mutator: &mut Mutator<VM>, _tls: VMWorkerT
 /// Release mutator. Do nothing.
 fn pp_mutator_release<VM: VMBinding>(_mutator: &mut Mutator<VM>, _tls: VMWorkerThread) {}
 
+const RESERVED_ALLOCATORS: ReservedAllocators = ReservedAllocators {
+    n_large_object: 1,
+    ..ReservedAllocators::DEFAULT
+};
+
 lazy_static! {
-    pub static ref ALLOCATOR_MAPPING: EnumMap<AllocationType, AllocatorSelector> = enum_map! {
-        AllocationType::Default | AllocationType::Los => AllocatorSelector::LargeObject(0),
-        // Temporarily put code and readonly objects to immortal space, for v8 support.
-        AllocationType::Immortal | AllocationType::Code | AllocationType::LargeCode | AllocationType::ReadOnly => AllocatorSelector::BumpPointer(0),
+    pub static ref ALLOCATOR_MAPPING: EnumMap<AllocationSemantics, AllocatorSelector> = {
+        let mut map = create_allocator_mapping(RESERVED_ALLOCATORS, true);
+        map[AllocationSemantics::Default] = AllocatorSelector::LargeObject(0);
+        map
     };
 }
 
@@ -35,13 +42,11 @@ pub fn create_pp_mutator<VM: VMBinding>(
     let page = plan.downcast_ref::<PageProtect<VM>>().unwrap();
     let config = MutatorConfig {
         allocator_mapping: &*ALLOCATOR_MAPPING,
-        space_mapping: box vec![
-            (
-                AllocatorSelector::BumpPointer(0),
-                page.common.get_immortal(),
-            ),
-            (AllocatorSelector::LargeObject(0), &page.space),
-        ],
+        space_mapping: box {
+            let mut vec = create_space_mapping(RESERVED_ALLOCATORS, true, plan);
+            vec.push((AllocatorSelector::LargeObject(0), &page.space));
+            vec
+        },
         prepare_func: &pp_mutator_prepare,
         release_func: &pp_mutator_release,
     };
