@@ -1,8 +1,10 @@
 use crate::plan::TransitiveClosure;
-use crate::plan::{AllocationSemantics, CopyContext};
+use crate::policy::copy_context::PolicyCopyContext;
 use crate::policy::space::SpaceOptions;
 use crate::policy::space::{CommonSpace, Space, SFT};
+use crate::scheduler::GCWorker;
 use crate::util::constants::CARD_META_PAGES_PER_REGION;
+use crate::util::copy::*;
 use crate::util::heap::layout::heap_layout::{Mmapper, VMMap};
 #[cfg(feature = "global_alloc_bit")]
 use crate::util::heap::layout::vm_layout_constants::BYTES_IN_CHUNK;
@@ -32,7 +34,7 @@ impl<VM: VMBinding> SFT for CopySpace<VM> {
     }
 
     fn is_live(&self, object: ObjectReference) -> bool {
-        !self.from_space() || object_forwarding::is_forwarded::<VM>(object)
+        !self.is_from_space() || object_forwarding::is_forwarded::<VM>(object)
     }
 
     fn is_movable(&self) -> bool {
@@ -41,7 +43,7 @@ impl<VM: VMBinding> SFT for CopySpace<VM> {
 
     #[cfg(feature = "sanity")]
     fn is_sane(&self) -> bool {
-        !self.from_space()
+        !self.is_from_space()
     }
 
     fn initialize_object_metadata(&self, _object: ObjectReference, _alloc: bool) {
@@ -51,7 +53,7 @@ impl<VM: VMBinding> SFT for CopySpace<VM> {
 
     #[inline(always)]
     fn get_forwarded_object(&self, object: ObjectReference) -> Option<ObjectReference> {
-        if !self.from_space() {
+        if !self.is_from_space() {
             return None;
         }
 
@@ -177,21 +179,21 @@ impl<VM: VMBinding> CopySpace<VM> {
         }
     }
 
-    fn from_space(&self) -> bool {
+    fn is_from_space(&self) -> bool {
         self.from_space.load(Ordering::SeqCst)
     }
 
     #[inline]
-    pub fn trace_object<T: TransitiveClosure, C: CopyContext>(
+    pub fn trace_object<T: TransitiveClosure>(
         &self,
         trace: &mut T,
         object: ObjectReference,
-        semantics: AllocationSemantics,
-        copy_context: &mut C,
+        semantics: CopySemantics,
+        worker: &mut GCWorker<VM>,
     ) -> ObjectReference {
         trace!("copyspace.trace_object(, {:?}, {:?})", object, semantics,);
         debug_assert!(
-            self.from_space(),
+            self.is_from_space(),
             "Trace object called for object ({:?}) in to-space",
             object
         );
@@ -215,8 +217,11 @@ impl<VM: VMBinding> CopySpace<VM> {
             new_object
         } else {
             trace!("... no it isn't. Copying");
-            let new_object =
-                object_forwarding::forward_object::<VM, _>(object, semantics, copy_context);
+            let new_object = object_forwarding::forward_object::<VM>(
+                object,
+                semantics,
+                worker.get_copy_context_mut(),
+            );
             trace!("Forwarding pointer");
             trace.process_node(new_object);
             trace!("Copied [{:?} -> {:?}]", object, new_object);
@@ -258,3 +263,51 @@ impl<VM: VMBinding> CopySpace<VM> {
         trace!("Unprotect {:x} {:x}", start, start + extent);
     }
 }
+
+use crate::plan::Plan;
+use crate::util::alloc::Allocator;
+use crate::util::alloc::BumpAllocator;
+use crate::util::opaque_pointer::VMWorkerThread;
+
+/// Copy allocator for CopySpace
+pub struct CopySpaceCopyContext<VM: VMBinding> {
+    copy_allocator: BumpAllocator<VM>,
+}
+
+impl<VM: VMBinding> PolicyCopyContext for CopySpaceCopyContext<VM> {
+    type VM = VM;
+
+    fn prepare(&mut self) {}
+
+    fn release(&mut self) {}
+
+    #[inline(always)]
+    fn alloc_copy(
+        &mut self,
+        _original: ObjectReference,
+        bytes: usize,
+        align: usize,
+        offset: isize,
+    ) -> Address {
+        self.copy_allocator.alloc(bytes, align, offset)
+    }
+}
+
+impl<VM: VMBinding> CopySpaceCopyContext<VM> {
+    pub fn new(
+        tls: VMWorkerThread,
+        plan: &'static dyn Plan<VM = VM>,
+        tospace: &'static CopySpace<VM>,
+    ) -> Self {
+        CopySpaceCopyContext {
+            copy_allocator: BumpAllocator::new(tls.0, tospace, plan),
+        }
+    }
+}
+
+impl<VM: VMBinding> CopySpaceCopyContext<VM> {
+    pub fn rebind(&mut self, space: &CopySpace<VM>) {
+        self.copy_allocator
+            .rebind(unsafe { &*{ space as *const _ } });
+    }
+}
