@@ -49,9 +49,8 @@ impl<C: GCWorkContext + 'static> GCWork<C::VM> for Prepare<C> {
             mmtk.scheduler.work_buckets[WorkBucketStage::Prepare]
                 .add(PrepareMutator::<C::VM>::new(mutator));
         }
-        for w in &mmtk.scheduler.worker_group().workers {
-            w.local_work_bucket
-                .add(PrepareCollector::<C::CopyContextType>::new());
+        for w in &mmtk.scheduler.workers_shared {
+            w.local_work_bucket.add(PrepareCollector);
         }
     }
 }
@@ -78,18 +77,13 @@ impl<VM: VMBinding> GCWork<VM> for PrepareMutator<VM> {
 
 /// The collector GC Preparation Work
 #[derive(Default)]
-pub struct PrepareCollector<W: CopyContext + GCWorkerLocal>(PhantomData<W>);
+pub struct PrepareCollector;
 
-impl<W: CopyContext + GCWorkerLocal> PrepareCollector<W> {
-    pub fn new() -> Self {
-        PrepareCollector(PhantomData)
-    }
-}
-
-impl<VM: VMBinding, W: CopyContext + GCWorkerLocal> GCWork<VM> for PrepareCollector<W> {
-    fn do_work(&mut self, worker: &mut GCWorker<VM>, _mmtk: &'static MMTK<VM>) {
+impl<VM: VMBinding> GCWork<VM> for PrepareCollector {
+    fn do_work(&mut self, worker: &mut GCWorker<VM>, mmtk: &'static MMTK<VM>) {
         trace!("Prepare Collector");
-        unsafe { worker.local::<W>() }.prepare();
+        worker.get_copy_context_mut().prepare();
+        mmtk.plan.prepare_worker(worker);
     }
 }
 
@@ -123,9 +117,8 @@ impl<C: GCWorkContext + 'static> GCWork<C::VM> for Release<C> {
             mmtk.scheduler.work_buckets[WorkBucketStage::Release]
                 .add(ReleaseMutator::<C::VM>::new(mutator));
         }
-        for w in &mmtk.scheduler.worker_group().workers {
-            w.local_work_bucket
-                .add(ReleaseCollector::<C::CopyContextType>::new());
+        for w in &mmtk.scheduler.workers_shared {
+            w.local_work_bucket.add(ReleaseCollector);
         }
         // TODO: Process weak references properly
         mmtk.reference_processors.clear();
@@ -154,18 +147,12 @@ impl<VM: VMBinding> GCWork<VM> for ReleaseMutator<VM> {
 
 /// The collector release Work
 #[derive(Default)]
-pub struct ReleaseCollector<W: CopyContext + GCWorkerLocal>(PhantomData<W>);
+pub struct ReleaseCollector;
 
-impl<W: CopyContext + GCWorkerLocal> ReleaseCollector<W> {
-    pub fn new() -> Self {
-        ReleaseCollector(PhantomData)
-    }
-}
-
-impl<VM: VMBinding, W: CopyContext + GCWorkerLocal> GCWork<VM> for ReleaseCollector<W> {
+impl<VM: VMBinding> GCWork<VM> for ReleaseCollector {
     fn do_work(&mut self, worker: &mut GCWorker<VM>, _mmtk: &'static MMTK<VM>) {
         trace!("Release Collector");
-        unsafe { worker.local::<W>() }.release();
+        worker.get_copy_context_mut().release();
     }
 }
 
@@ -336,22 +323,22 @@ impl<E: ProcessEdgesWork> GCWork<E::VM> for ScanVMSpecificRoots<E> {
     }
 }
 
-pub struct ProcessEdgesBase<E: ProcessEdgesWork> {
+pub struct ProcessEdgesBase<VM: VMBinding> {
     pub edges: Vec<Address>,
     pub nodes: Vec<ObjectReference>,
-    mmtk: &'static MMTK<E::VM>,
+    mmtk: &'static MMTK<VM>,
     // Use raw pointer for fast pointer dereferencing, instead of using `Option<&'static mut GCWorker<E::VM>>`.
     // Because a copying gc will dereference this pointer at least once for every object copy.
-    worker: *mut GCWorker<E::VM>,
+    worker: *mut GCWorker<VM>,
     pub roots: bool,
 }
 
-unsafe impl<E: ProcessEdgesWork> Send for ProcessEdgesBase<E> {}
+unsafe impl<VM: VMBinding> Send for ProcessEdgesBase<VM> {}
 
-impl<E: ProcessEdgesWork> ProcessEdgesBase<E> {
+impl<VM: VMBinding> ProcessEdgesBase<VM> {
     // Requires an MMTk reference. Each plan-specific type that uses ProcessEdgesBase can get a static plan reference
     // at creation. This avoids overhead for dynamic dispatch or downcasting plan for each object traced.
-    pub fn new(edges: Vec<Address>, roots: bool, mmtk: &'static MMTK<E::VM>) -> Self {
+    pub fn new(edges: Vec<Address>, roots: bool, mmtk: &'static MMTK<VM>) -> Self {
         #[cfg(feature = "extreme_assertions")]
         if crate::util::edge_logger::should_check_duplicate_edges(&*mmtk.plan) {
             for edge in &edges {
@@ -367,19 +354,19 @@ impl<E: ProcessEdgesWork> ProcessEdgesBase<E> {
             roots,
         }
     }
-    pub fn set_worker(&mut self, worker: &mut GCWorker<E::VM>) {
+    pub fn set_worker(&mut self, worker: &mut GCWorker<VM>) {
         self.worker = worker;
     }
     #[inline]
-    pub fn worker(&self) -> &'static mut GCWorker<E::VM> {
+    pub fn worker(&self) -> &'static mut GCWorker<VM> {
         unsafe { &mut *self.worker }
     }
     #[inline]
-    pub fn mmtk(&self) -> &'static MMTK<E::VM> {
+    pub fn mmtk(&self) -> &'static MMTK<VM> {
         self.mmtk
     }
     #[inline]
-    pub fn plan(&self) -> &'static dyn Plan<VM = E::VM> {
+    pub fn plan(&self) -> &'static dyn Plan<VM = VM> {
         &*self.mmtk.plan
     }
     /// Pop all nodes from nodes, and clear nodes to an empty vector.
@@ -397,7 +384,7 @@ impl<E: ProcessEdgesWork> ProcessEdgesBase<E> {
 
 /// Scan & update a list of object slots
 pub trait ProcessEdgesWork:
-    Send + 'static + Sized + DerefMut + Deref<Target = ProcessEdgesBase<Self>>
+    Send + 'static + Sized + DerefMut + Deref<Target = ProcessEdgesBase<Self::VM>>
 {
     type VM: VMBinding;
 
