@@ -1,15 +1,17 @@
 use super::stat::SchedulerStat;
+use super::work_bucket::WorkBucketStage::*;
 use super::work_bucket::*;
-use super::worker::{GCWorker, WorkerGroup};
+use super::worker::{GCWorker, GCWorkerShared};
 use super::*;
 use crate::mmtk::MMTK;
 use crate::util::opaque_pointer::*;
-use crate::vm::VMBinding;
+use crate::vm::Collection;
+use crate::vm::{GCThreadContext, VMBinding};
 use enum_map::{enum_map, EnumMap};
 use std::collections::HashMap;
 use std::sync::atomic::Ordering;
-use std::sync::mpsc::{channel, Receiver, Sender};
-use std::sync::{Arc, Condvar, Mutex, RwLock};
+use std::sync::mpsc::channel;
+use std::sync::{Arc, Condvar, Mutex};
 
 pub enum CoordinatorMessage<VM: VMBinding> {
     Work(Box<dyn CoordinatorWork<VM>>),
@@ -17,23 +19,18 @@ pub enum CoordinatorMessage<VM: VMBinding> {
     BucketDrained,
 }
 
+/// The shared data structure for distributing work packets between worker threads and the coordinator thread.
 pub struct GCWorkScheduler<VM: VMBinding> {
+    /// Work buckets for worker threads
     pub work_buckets: EnumMap<WorkBucketStage, WorkBucket<VM>>,
     /// Work for the coordinator thread
     pub coordinator_work: WorkBucket<VM>,
-    /// workers
-    worker_group: Option<Arc<WorkerGroup<VM>>>,
+    /// The shared parts of GC workers
+    pub workers_shared: Vec<Arc<GCWorkerShared<VM>>>,
+    /// The shared part of the GC worker object of the controller thread
+    coordinator_worker_shared: Arc<GCWorkerShared<VM>>,
     /// Condition Variable for worker synchronization
     pub worker_monitor: Arc<(Mutex<()>, Condvar)>,
-    mmtk: Option<&'static MMTK<VM>>,
-    coordinator_worker: Option<RwLock<GCWorker<VM>>>,
-    /// A message channel to send new coordinator work and other actions to the coordinator thread
-    channel: (
-        Sender<CoordinatorMessage<VM>>,
-        Receiver<CoordinatorMessage<VM>>,
-    ),
-    startup: Mutex<Option<Box<dyn CoordinatorWork<VM>>>>,
-    finalizer: Mutex<Option<Box<dyn CoordinatorWork<VM>>>>,
     /// A callback to be fired after the `Closure` bucket is drained.
     /// This callback should return `true` if it adds more work packets to the
     /// `Closure` bucket. `WorkBucket::can_open` then consult this return value
@@ -46,75 +43,29 @@ pub struct GCWorkScheduler<VM: VMBinding> {
     closure_end: Mutex<Option<Box<dyn Send + Fn() -> bool>>>,
 }
 
-// The 'channel' inside Scheduler disallows Sync for Scheduler. We have to make sure we use channel properly:
-// 1. We should never directly use Sender. We clone the sender and let each worker have their own copy.
-// 2. Only the coordinator can use Receiver.
-// TODO: We should remove channel from Scheduler, and directly send Sender/Receiver when creating the coordinator and
-// the workers.
+// FIXME: GCWorkScheduler should be naturally Sync, but we cannot remove this `impl` yet.
+// Some subtle interaction between ObjectRememberingBarrier, Mutator and some GCWork instances
+// makes the compiler think WorkBucket is not Sync.
 unsafe impl<VM: VMBinding> Sync for GCWorkScheduler<VM> {}
 
 impl<VM: VMBinding> GCWorkScheduler<VM> {
-    pub fn new() -> Arc<Self> {
+    pub fn new(num_workers: usize) -> Arc<Self> {
         let worker_monitor: Arc<(Mutex<()>, Condvar)> = Default::default();
-        Arc::new(Self {
-            work_buckets: enum_map! {
-                WorkBucketStage::Unconstrained => WorkBucket::new(true, worker_monitor.clone()),
-                WorkBucketStage::Prepare => WorkBucket::new(false, worker_monitor.clone()),
-                WorkBucketStage::Closure => WorkBucket::new(false, worker_monitor.clone()),
-                WorkBucketStage::RefClosure => WorkBucket::new(false, worker_monitor.clone()),
-                WorkBucketStage::CalculateForwarding => WorkBucket::new(false, worker_monitor.clone()),
-                WorkBucketStage::RefForwarding => WorkBucket::new(false, worker_monitor.clone()),
-                WorkBucketStage::Compact => WorkBucket::new(false, worker_monitor.clone()),
-                WorkBucketStage::Release => WorkBucket::new(false, worker_monitor.clone()),
-                WorkBucketStage::Final => WorkBucket::new(false, worker_monitor.clone()),
-            },
-            coordinator_work: WorkBucket::new(true, worker_monitor.clone()),
-            worker_group: None,
-            worker_monitor,
-            mmtk: None,
-            coordinator_worker: None,
-            channel: channel(),
-            startup: Mutex::new(None),
-            finalizer: Mutex::new(None),
-            closure_end: Mutex::new(None),
-        })
-    }
 
-    #[inline]
-    pub fn num_workers(&self) -> usize {
-        self.worker_group.as_ref().unwrap().worker_count()
-    }
-
-    pub fn initialize(
-        self: &'static Arc<Self>,
-        num_workers: usize,
-        mmtk: &'static MMTK<VM>,
-        tls: VMThread,
-    ) {
-        use crate::scheduler::work_bucket::WorkBucketStage::*;
-        let num_workers = if cfg!(feature = "single_worker") {
-            1
-        } else {
-            num_workers
+        // Create work buckets for workers.
+        let mut work_buckets = enum_map! {
+            WorkBucketStage::Unconstrained => WorkBucket::new(true, worker_monitor.clone()),
+            WorkBucketStage::Prepare => WorkBucket::new(false, worker_monitor.clone()),
+            WorkBucketStage::Closure => WorkBucket::new(false, worker_monitor.clone()),
+            WorkBucketStage::RefClosure => WorkBucket::new(false, worker_monitor.clone()),
+            WorkBucketStage::CalculateForwarding => WorkBucket::new(false, worker_monitor.clone()),
+            WorkBucketStage::RefForwarding => WorkBucket::new(false, worker_monitor.clone()),
+            WorkBucketStage::Compact => WorkBucket::new(false, worker_monitor.clone()),
+            WorkBucketStage::Release => WorkBucket::new(false, worker_monitor.clone()),
+            WorkBucketStage::Final => WorkBucket::new(false, worker_monitor.clone()),
         };
 
-        let mut self_mut = self.clone();
-        let self_mut = unsafe { Arc::get_mut_unchecked(&mut self_mut) };
-
-        self_mut.mmtk = Some(mmtk);
-        self_mut.coordinator_worker = Some(RwLock::new(GCWorker::new(
-            0,
-            Arc::downgrade(self),
-            true,
-            self.channel.0.clone(),
-        )));
-        self_mut.worker_group = Some(WorkerGroup::new(
-            num_workers,
-            Arc::downgrade(self),
-            self.channel.0.clone(),
-        ));
-        self.worker_group.as_ref().unwrap().spawn_workers(tls, mmtk);
-
+        // Set the open condition of each bucket.
         {
             // Unconstrained is always open. Prepare will be opened at the beginning of a GC.
             // This vec will grow for each stage we call with open_next()
@@ -122,12 +73,12 @@ impl<VM: VMBinding> GCWorkScheduler<VM> {
             // The rest will open after the previous stage is done.
             let mut open_next = |s: WorkBucketStage| {
                 let cur_stages = open_stages.clone();
-                self_mut.work_buckets[s].set_open_condition(move || {
-                    let should_open =
-                        self.are_buckets_drained(&cur_stages) && self.worker_group().all_parked();
+                work_buckets[s].set_open_condition(move |scheduler: &GCWorkScheduler<VM>| {
+                    let should_open = scheduler.are_buckets_drained(&cur_stages)
+                        && scheduler.all_workers_parked();
                     // Additional check before the `RefClosure` bucket opens.
                     if should_open && s == WorkBucketStage::RefClosure {
-                        if let Some(closure_end) = self.closure_end.lock().unwrap().as_ref() {
+                        if let Some(closure_end) = scheduler.closure_end.lock().unwrap().as_ref() {
                             if closure_end() {
                                 // Don't open `RefClosure` if `closure_end` added more works to `Closure`.
                                 return false;
@@ -147,6 +98,73 @@ impl<VM: VMBinding> GCWorkScheduler<VM> {
             open_next(Release);
             open_next(Final);
         }
+
+        // Create the work bucket for the controller.
+        let coordinator_work = WorkBucket::new(true, worker_monitor.clone());
+
+        // We prepare the shared part of workers, but do not create the actual workers now.
+        // The shared parts of workers are communication hubs between controller and workers.
+        let workers_shared = (0..num_workers)
+            .map(|_| Arc::new(GCWorkerShared::<VM>::new(worker_monitor.clone())))
+            .collect::<Vec<_>>();
+
+        // Similarly, we create the shared part of the work of the controller, but not the controller itself.
+        let coordinator_worker_shared = Arc::new(GCWorkerShared::<VM>::new(worker_monitor.clone()));
+
+        Arc::new(Self {
+            work_buckets,
+            coordinator_work,
+            workers_shared,
+            coordinator_worker_shared,
+            worker_monitor,
+            closure_end: Mutex::new(None),
+        })
+    }
+
+    #[inline]
+    pub fn num_workers(&self) -> usize {
+        self.workers_shared.len()
+    }
+
+    pub fn all_workers_parked(&self) -> bool {
+        self.workers_shared.iter().all(|w| w.is_parked())
+    }
+
+    /// Create GC threads, including the controller thread and all workers.
+    pub fn spawn_gc_threads(self: &Arc<Self>, mmtk: &'static MMTK<VM>, tls: VMThread) {
+        // Create the communication channel.
+        let (sender, receiver) = channel::<CoordinatorMessage<VM>>();
+
+        // Spawn the controller thread.
+        let coordinator_worker = GCWorker::new(
+            mmtk,
+            0,
+            self.clone(),
+            true,
+            sender.clone(),
+            self.coordinator_worker_shared.clone(),
+        );
+        let gc_controller = GCController::new(
+            mmtk,
+            mmtk.plan.base().gc_requester.clone(),
+            self.clone(),
+            receiver,
+            coordinator_worker,
+        );
+        VM::VMCollection::spawn_gc_thread(tls, GCThreadContext::<VM>::Controller(gc_controller));
+
+        // Spawn each worker thread.
+        for (ordinal, shared) in self.workers_shared.iter().enumerate() {
+            let worker = Box::new(GCWorker::new(
+                mmtk,
+                ordinal,
+                self.clone(),
+                false,
+                sender.clone(),
+                shared.clone(),
+            ));
+            VM::VMCollection::spawn_gc_thread(tls, GCThreadContext::<VM>::Worker(worker));
+        }
     }
 
     /// Schedule all the common work packets
@@ -182,13 +200,11 @@ impl<VM: VMBinding> GCWorkScheduler<VM> {
         {
             use crate::util::sanity::sanity_checker::ScheduleSanityGC;
             self.work_buckets[WorkBucketStage::Final]
-                .add(ScheduleSanityGC::<C::PlanType, C::CopyContextType>::new(
-                    plan,
-                ));
+                .add(ScheduleSanityGC::<C::PlanType>::new(plan));
         }
 
         // Finalization
-        if !plan.base().options.no_finalizer {
+        if !*plan.base().options.no_finalizer {
             use crate::util::finalizable_processor::{Finalization, ForwardFinalization};
             // finalization
             self.work_buckets[WorkBucketStage::RefClosure]
@@ -199,48 +215,28 @@ impl<VM: VMBinding> GCWorkScheduler<VM> {
                     .add(ForwardFinalization::<C::ProcessEdgesWorkType>::new());
             }
         }
-
-        // Set EndOfGC to run at the end
-        self.set_finalizer(Some(EndOfGC));
     }
 
     fn are_buckets_drained(&self, buckets: &[WorkBucketStage]) -> bool {
         buckets.iter().all(|&b| self.work_buckets[b].is_drained())
     }
 
-    pub fn initialize_worker(self: &Arc<Self>, tls: VMWorkerThread) {
-        let mut coordinator_worker = self.coordinator_worker.as_ref().unwrap().write().unwrap();
-        coordinator_worker.init(tls);
-    }
-
-    pub fn set_initializer<W: CoordinatorWork<VM>>(&self, w: Option<W>) {
-        *self.startup.lock().unwrap() = w.map(|w| box w as Box<dyn CoordinatorWork<VM>>);
-    }
-
-    pub fn set_finalizer<W: CoordinatorWork<VM>>(&self, w: Option<W>) {
-        *self.finalizer.lock().unwrap() = w.map(|w| box w as Box<dyn CoordinatorWork<VM>>);
-    }
-
     pub fn on_closure_end(&self, f: Box<dyn Send + Fn() -> bool>) {
         *self.closure_end.lock().unwrap() = Some(f);
     }
 
-    pub fn worker_group(&self) -> Arc<WorkerGroup<VM>> {
-        self.worker_group.as_ref().unwrap().clone()
-    }
-
-    fn all_buckets_empty(&self) -> bool {
+    pub fn all_buckets_empty(&self) -> bool {
         self.work_buckets.values().all(|bucket| bucket.is_empty())
     }
 
     /// Open buckets if their conditions are met
-    fn update_buckets(&self) {
+    pub fn update_buckets(&self) {
         let mut buckets_updated = false;
         for (id, bucket) in self.work_buckets.iter() {
             if id == WorkBucketStage::Unconstrained {
                 continue;
             }
-            buckets_updated |= bucket.update();
+            buckets_updated |= bucket.update(self);
         }
         if buckets_updated {
             // Notify the workers for new work
@@ -249,58 +245,6 @@ impl<VM: VMBinding> GCWorkScheduler<VM> {
         }
     }
 
-    /// Execute coordinator work, in the controller thread
-    fn process_coordinator_work(&self, mut work: Box<dyn CoordinatorWork<VM>>) {
-        let mut coordinator_worker = self.coordinator_worker.as_ref().unwrap().write().unwrap();
-        let mmtk = self.mmtk.unwrap();
-        work.do_work_with_stat(&mut coordinator_worker, mmtk);
-    }
-
-    /// Drain the message queue and execute coordinator work. Only the coordinator should call this.
-    pub fn wait_for_completion(&self) {
-        // At the start of a GC, we probably already have received a `ScheduleCollection` work. Run it now.
-        if let Some(initializer) = self.startup.lock().unwrap().take() {
-            self.process_coordinator_work(initializer);
-        }
-        loop {
-            let message = self.channel.1.recv().unwrap();
-            match message {
-                CoordinatorMessage::Work(work) => {
-                    self.process_coordinator_work(work);
-                }
-                CoordinatorMessage::AllWorkerParked | CoordinatorMessage::BucketDrained => {
-                    self.update_buckets();
-                }
-            }
-            let _guard = self.worker_monitor.0.lock().unwrap();
-            if self.worker_group().all_parked() && self.all_buckets_empty() {
-                break;
-            }
-        }
-        for message in self.channel.1.try_iter() {
-            if let CoordinatorMessage::Work(work) = message {
-                self.process_coordinator_work(work);
-            }
-        }
-        self.deactivate_all();
-        // Finalization: Resume mutators, reset gc states
-        // Note: Resume-mutators must happen after all work buckets are closed.
-        //       Otherwise, for generational GCs, workers will receive and process
-        //       newly generated remembered-sets from those open buckets.
-        //       But these remsets should be preserved until next GC.
-        if let Some(finalizer) = self.finalizer.lock().unwrap().take() {
-            self.process_coordinator_work(finalizer);
-        }
-        debug_assert!(!self.work_buckets[WorkBucketStage::Prepare].is_activated());
-        debug_assert!(!self.work_buckets[WorkBucketStage::Closure].is_activated());
-        debug_assert!(!self.work_buckets[WorkBucketStage::RefClosure].is_activated());
-        debug_assert!(!self.work_buckets[WorkBucketStage::CalculateForwarding].is_activated());
-        debug_assert!(!self.work_buckets[WorkBucketStage::RefForwarding].is_activated());
-        debug_assert!(!self.work_buckets[WorkBucketStage::Compact].is_activated());
-        debug_assert!(!self.work_buckets[WorkBucketStage::Release].is_activated());
-        debug_assert!(!self.work_buckets[WorkBucketStage::Final].is_activated());
-    }
-
     pub fn deactivate_all(&self) {
         self.work_buckets[WorkBucketStage::Prepare].deactivate();
         self.work_buckets[WorkBucketStage::Closure].deactivate();
@@ -323,6 +267,17 @@ impl<VM: VMBinding> GCWorkScheduler<VM> {
         self.work_buckets[WorkBucketStage::Final].deactivate();
     }
 
+    pub fn debug_assert_all_buckets_deactivated(&self) {
+        debug_assert!(!self.work_buckets[WorkBucketStage::Prepare].is_activated());
+        debug_assert!(!self.work_buckets[WorkBucketStage::Closure].is_activated());
+        debug_assert!(!self.work_buckets[WorkBucketStage::RefClosure].is_activated());
+        debug_assert!(!self.work_buckets[WorkBucketStage::CalculateForwarding].is_activated());
+        debug_assert!(!self.work_buckets[WorkBucketStage::RefForwarding].is_activated());
+        debug_assert!(!self.work_buckets[WorkBucketStage::Compact].is_activated());
+        debug_assert!(!self.work_buckets[WorkBucketStage::Release].is_activated());
+        debug_assert!(!self.work_buckets[WorkBucketStage::Final].is_activated());
+    }
+
     pub fn add_coordinator_work(&self, work: impl CoordinatorWork<VM>, worker: &GCWorker<VM>) {
         worker
             .sender
@@ -332,8 +287,8 @@ impl<VM: VMBinding> GCWorkScheduler<VM> {
 
     #[inline]
     fn pop_scheduable_work(&self, worker: &GCWorker<VM>) -> Option<(Box<dyn GCWork<VM>>, bool)> {
-        if let Some(work) = worker.local_work_bucket.poll() {
-            return Some((work, worker.local_work_bucket.is_empty()));
+        if let Some(work) = worker.shared.local_work_bucket.poll() {
+            return Some((work, worker.shared.local_work_bucket.is_empty()));
         }
         for work_bucket in self.work_buckets.values() {
             if let Some(work) = work_bucket.poll() {
@@ -362,10 +317,10 @@ impl<VM: VMBinding> GCWorkScheduler<VM> {
 
     #[cold]
     fn poll_slow(&self, worker: &GCWorker<VM>) -> Box<dyn GCWork<VM>> {
-        debug_assert!(!worker.is_parked());
+        debug_assert!(!worker.shared.is_parked());
         let mut guard = self.worker_monitor.0.lock().unwrap();
         loop {
-            debug_assert!(!worker.is_parked());
+            debug_assert!(!worker.shared.is_parked());
             if let Some((work, bucket_is_empty)) = self.pop_scheduable_work(worker) {
                 if bucket_is_empty {
                     worker
@@ -376,8 +331,8 @@ impl<VM: VMBinding> GCWorkScheduler<VM> {
                 return work;
             }
             // Park this worker
-            worker.parked.store(true, Ordering::SeqCst);
-            if self.worker_group().all_parked() {
+            worker.shared.parked.store(true, Ordering::SeqCst);
+            if self.all_workers_parked() {
                 worker
                     .sender
                     .send(CoordinatorMessage::AllWorkerParked)
@@ -386,30 +341,32 @@ impl<VM: VMBinding> GCWorkScheduler<VM> {
             // Wait
             guard = self.worker_monitor.1.wait(guard).unwrap();
             // Unpark this worker
-            worker.parked.store(false, Ordering::SeqCst);
+            worker.shared.parked.store(false, Ordering::SeqCst);
         }
     }
 
     pub fn enable_stat(&self) {
-        for worker in &self.worker_group().workers {
-            worker.stat.enable();
+        for worker in &self.workers_shared {
+            let worker_stat = worker.borrow_stat();
+            worker_stat.enable();
         }
-        let coordinator_worker = self.coordinator_worker.as_ref().unwrap().read().unwrap();
-        coordinator_worker.stat.enable();
+        let coordinator_worker_stat = self.coordinator_worker_shared.borrow_stat();
+        coordinator_worker_stat.enable();
     }
 
     pub fn statistics(&self) -> HashMap<String, String> {
         let mut summary = SchedulerStat::default();
-        for worker in &self.worker_group().workers {
-            summary.merge(&worker.stat);
+        for worker in &self.workers_shared {
+            let worker_stat = worker.borrow_stat();
+            summary.merge(&worker_stat);
         }
-        let coordinator_worker = self.coordinator_worker.as_ref().unwrap().read().unwrap();
-        summary.merge(&coordinator_worker.stat);
+        let coordinator_worker_stat = self.coordinator_worker_shared.borrow_stat();
+        summary.merge(&coordinator_worker_stat);
         summary.harness_stat()
     }
 
     pub fn notify_mutators_paused(&self, mmtk: &'static MMTK<VM>) {
-        mmtk.plan.base().control_collector_context.clear_request();
+        mmtk.plan.base().gc_requester.clear_request();
         debug_assert!(!self.work_buckets[WorkBucketStage::Prepare].is_activated());
         self.work_buckets[WorkBucketStage::Prepare].activate();
         let _guard = self.worker_monitor.0.lock().unwrap();
