@@ -1,5 +1,6 @@
 pub(crate) mod allocator;
 pub use allocator::fill_alignment_gap;
+pub use allocator::AllocationError;
 pub use allocator::Allocator;
 
 pub(crate) mod allocators;
@@ -20,6 +21,4 @@ pub use self::immix_allocator::ImmixAllocator;
 mod markcompact_allocator;
 pub use markcompact_allocator::MarkCompactAllocator;
 
-pub(crate) mod dump_linear_scan;
 pub(crate) mod embedded_meta_data;
-pub(crate) mod linear_scan;
