@@ -15,6 +15,10 @@ pub mod alloc;
 pub mod constants;
 /// Calculation, conversion and rounding for memory related numbers.
 pub mod conversions;
+/// The copy allocators for a GC worker.
+pub mod copy;
+/// Linear scan through a heap range
+pub mod linear_scan;
 /// Wrapper functions for memory syscalls such as mmap, mprotect, etc.
 pub mod memory;
 /// Opaque pointers used in MMTk, e.g. VMThread.
