@@ -1,12 +1,12 @@
 use atomic::Ordering;
 
 use self::specs::*;
-use crate::plan::AllocationSemantics;
-use crate::plan::CopyContext;
 use crate::util::metadata::header_metadata::HeaderMetadataSpec;
 use crate::util::{Address, ObjectReference};
 use crate::vm::VMBinding;
 
+use crate::util::copy::*;
+
 /// VM-specific methods for object model.
 ///
 /// This trait includes 3 parts:
@@ -181,16 +181,17 @@ pub trait ObjectModel<VM: VMBinding> {
     ) -> usize;
 
     /// Copy an object and return the address of the new object. Usually in the implementation of this method,
-    /// `alloc_copy()` and `post_copy()` from a plan's [`CopyContext`](../trait.CopyContext.html) are used for copying.
+    /// `alloc_copy()` and `post_copy()` from [`GCWorkerCopyContext`](util/copy/struct.GCWorkerCopyContext.html)
+    /// are used for copying.
     ///
     /// Arguments:
     /// * `from`: The address of the object to be copied.
-    /// * `semantics`: The allocation semantic to use.
-    /// * `copy_context`: The `CopyContext` for the GC thread.
+    /// * `semantics`: The copy semantic to use.
+    /// * `copy_context`: The `GCWorkerCopyContext` for the GC thread.
     fn copy(
         from: ObjectReference,
-        semantics: AllocationSemantics,
-        copy_context: &mut impl CopyContext,
+        semantics: CopySemantics,
+        copy_context: &mut GCWorkerCopyContext<VM>,
     ) -> ObjectReference;
 
     /// Copy an object. This is required
